#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ---------------------------------------------------------------------
# Simple logging client for tempsensor_datalogger
# 
# ---------------------------------------------------------------------
# Copyright (c) 2002-2012 boxtec internet appliances <playground@boxtec.ch>
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
# THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# --------------------------------------------------------------------------
#  $Id: simple_templogger.py,v 1.5 2012-01-09 00:12:36 obiwan Exp $
# --------------------------------------------------------------------------
# ###########################################################
# TODO: 
# NOTES:
# ###########################################################

"""
__author__  = "$Author: obiwan $ (cschnee@telemedia.ch)"
__appname__ = "simple_templogger.py"
__version__ = "$Revision: 1.5 $"
__date__    = "$Date: 2012-01-09 00:12:36 $"
__license__ = "BSD"
"""

################
# IMPORTS
################
import os, sys, serial, time, signal




#***************************************
# CONSTANTS
#***************************************
stl_release = '$Revision: 1.5 $'
stl_version = stl_release.replace('$','').replace('Revision: ','').strip()
logfile = 'data.log'
REPEAT_HEADERS = 50

#***************************************
# Functions
#***************************************
def writeblock(line):
    fh = open(logfile, 'a')
    fh.write(line)
    fh.close()
    
def writeheaders():
    hline = "#tlog, dtlog, dht11[T], dht22[T], dht33[T], ds18s20, thermistor1, thermistor2, tmp36 dht11[H], dht22[H], dht33[H], ext[T], ext[H]\n"
    fh = open(logfile, 'a')
    fh.write(hline)
    fh.close()
    
def input_timeout_handler(signum, frame):
    raise NameError("Timeout")
    
    
def loginput(prompt="", timeout=10):
    signal.signal(signal.SIGALRM, input_timeout_handler)
    signal.alarm(timeout)
    try:
        ext_data = raw_input(prompt)
        signal.alarm(0)
        return ext_data
    except KeyboardInterrupt:
        shutdown(0,0)
    except NameError:
        signal.alarm(0)
        return    

def shutdown(signum, frame):
    print "\nLogger wird beendet..."
    lsp.close()
    signal.alarm(0)
    sys.exit(0)

#***************************************
# MAIN namespace / Main Loop
#***************************************
if __name__ == '__main__':
    # Install signal handler
    signal.signal(signal.SIGINT, shutdown)
    # find serial port on cmd line or use first
    if len(sys.argv) > 1:
        PORT = sys.argv[1]
    else:
        PORT = 0
    writeheaders()
    counter = 0
    t0 = int(time.time())
    # open serial port
    lsp = serial.Serial(PORT, 9600, timeout=5)
    # waste a line
    lsp.readline()
    newblock = 0
    dht11 = 0
    ext_t = 23
    ext_h = 30
    while 1:
        if counter == REPEAT_HEADERS:
            writeheaders()
            counter = 0
        trun = int(time.time()) - t0
        dtlog = time.strftime("%Y %m %d %H %M %S")
        if newblock == 1:
            writeblock("%s %s %s %s %s %s %s %s %s %s %s %s %s\n" %(dtlog, dht11[1], dht22[1], dht33[1], ds18s20, thermistor1, thermistor2, tmp36, \
                        dht11[2], dht22[2], dht33[2], ext_t, ext_h))
            newblock = 0
            counter += 1
        logline = lsp.readline()
        #print logline.strip()
        vals = logline.split()
        if len(vals) == 3:
            #DHT Sensor read
            print "Sensor:", vals[0], vals[1], vals[2]
            if vals[0] == "DHT11":
                dht11 = [vals[0], vals[1], vals[2]]
            elif vals[0] == "DHT22":
                dht22 = [vals[0], vals[1], vals[2]]
            elif vals[0] == "DHT33":
                dht33 = [vals[0], vals[1], vals[2]]
        elif len(vals) == 2:
            #Other Sensor read
            print "Sensor:", vals[0], vals[1]
            if vals[0] == "DS18S20":
                ds18s20 = vals[1]
            elif vals[0] == "Thermistor1":
                thermistor1 = vals[1]
            elif vals[0] == "Thermistor2":
                thermistor2 = vals[1]
            elif vals[0] == "TMP36":
                tmp36 = vals[1]
        elif len(vals) == 1 and dht11 > 0:
            new_ext_t = loginput("\nExterne Temperaturmessung: ", 10)
            if new_ext_t is not None:
                ext_t = new_ext_t
                print "Sensor T-extern:", ext_t
            new_ext_h = loginput("\nExterne Feuchtemessung: ", 10)
            if new_ext_h is not None:
                ext_h = new_ext_h
                print "Sensor H-extern:", ext_h
            else:
                print "\n"
            newblock = 1;
        time.sleep(0.3)
    lsp.close()
    
    
    
    
    
    

