// Copyright (c) 2002-2012 boxtec internet appliances <playground@boxtec.ch>
//
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
// 3. The name of the author may not be used to endorse or promote products
//    derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
// THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// --------------------------------------------------------------------------
//  $Id: tempsensor_datalogger.pde,v 1.6 2012-01-08 23:40:08 obiwan Exp $
// --------------------------------------------------------------------------

#include <OneWire.h>
#include <DallasTemperature.h>
#include <dht.h>

dht DHT;

#define V_AREF 4.85
#define DHT11_PIN 2
#define DHT22_PIN 3
#define DHT33_PIN 4
#define ONE_WIRE_BUS 5
#define THERMISTOR1_PIN 1
#define THERMISTOR2_PIN 2
#define TMP36_PIN 5
#define DELAY 30*1000

// DS18S20 Temperature chip i/o
// Setup a oneWire instance to communicate with any OneWire devices (not just Maxim/Dallas temperature ICs)
OneWire oneWire(ONE_WIRE_BUS);
// Pass our oneWire reference to Dallas Temperature. 
DallasTemperature sensors(&oneWire);
// arrays to hold device address
DeviceAddress insideThermometer;

double h_val;
double t_val;


void setup()
{
  // set pin modes
  Serial.begin(9600);
  // init onewire network
  sensors.begin();
  if (!sensors.getAddress(insideThermometer, 0)) Serial.println("Unable to find address for Device 0"); 
  // set the resolution to 9 bit (Each Dallas/Maxim device is capable of several different resolutions)
  sensors.setResolution(insideThermometer, 9);
  analogReference(DEFAULT);
}


void loop()
{
  // READ DATA DHT11
  readDHT11(DHT11_PIN, &t_val, &h_val);
  Serial.print("DHT11\t");
  Serial.print(t_val);
  Serial.print("\t");
  Serial.println(h_val);

  // READ DATA DHT22
  readDHT22(DHT22_PIN, &t_val, &h_val);
  Serial.print("DHT22\t");
  Serial.print(t_val);
  Serial.print("\t");
  Serial.println(h_val);

  // READ DATA DHT33
  readDHT22(DHT33_PIN, &t_val, &h_val);
  Serial.print("DHT33\t");
  Serial.print(t_val);
  Serial.print("\t");
  Serial.println(h_val);

  // READ DATA DS18S20
  // call sensors.requestTemperatures() to issue a global temperature 
  // request to all devices on the bus
  Serial.print("DS18S20\t");
  Serial.println(readDS18x20(insideThermometer));
  
  // READ DATA Thermistor
  //int thermistorTemp = map(analogRead(1), 0, 1023, 0, 500);
  Serial.print("Thermistor1\t");
  Serial.println(Thermistor(analogRead(THERMISTOR1_PIN)));
  // wait a short moment to let things stabilize
  delay(25);
  Serial.print("Thermistor2\t");
  Serial.println(Thermistor(analogRead(THERMISTOR2_PIN)));
  delay(25);
  
  //TMP36
  Serial.print("TMP36\t");
  Serial.println(readTMP36(TMP36_PIN));

  //signal end of block  
  Serial.println(millis()/60000);
  delay(DELAY);
}


double readDS18x20(DeviceAddress deviceAddress) {
  sensors.requestTemperatures(); // Send the command to get temperatures
  return sensors.getTempC(deviceAddress);
}  


double Thermistor(int RawADC) {
  double tempC;
  tempC = log(((10240000/RawADC) - 10000));
  tempC = 1 / (0.001129148 + (0.000234125 * tempC) + (0.0000000876741 * tempC * tempC * tempC));
  tempC = tempC - 273.15;           // Convert Kelvin to Celcius
  return tempC;
}

double readDHT11(int DPIN, double* t_val, double* h_val) {
  int chk = DHT.read11(DPIN);
  switch (chk)
  {
    case 0: break;
    case -1: Serial.print("Checksum error,\t"); break;
    case -2: Serial.print("Time out error,\t"); break;
    default: Serial.print("Unknown error,\t"); break;
  }
  *t_val = DHT.temperature,1;
  *h_val = DHT.humidity,1;
}

void readDHT22(int DPIN, double* t_val, double* h_val) {
  int chk = DHT.read22(DPIN);
  switch (chk)
  {
    case 0: break;
    case -1: Serial.print("Checksum error,\t"); break;
    case -2: Serial.print("Time out error,\t"); break;
    default: Serial.print("Unknown error,\t"); break;
  }
  *t_val = DHT.temperature,1;
  *h_val = DHT.humidity,1;
}

float readTMP36(int APIN) {
  delay(150);
  float aread = analogRead(APIN);
  float voltage = aread * V_AREF / 1024;
  float tempC = (voltage - 0.5) * 100;
  return tempC;
}
